
;Input:
;	byte (MENU_FLAG)
;		bit 0 = 1 : display a title
;			0 : don't display a title
;		bit 1 = 1 : put "buffer space" between entries and left edge
;			0 : no "buffer space"
;		bit 2 = 1 : draw border
;			0 : don't draw border
;		bit 3 = 1 : insert "#: " before first 10 entries
;			0 : keep entries "as is"
;	byte (MENU_Y) = Y coordinate of menu
;	byte (MENU_X) = X coordinate of menu
;	byte (MENU_WIDTH) = width of menu
;	byte (MENU_HEIGHT) = height of menu (number of displayed entries)
;	byte (MENU_LENGTH) = number of entries in menu
;	byte (MENU_ABSCUR) = starting absolute selection
;	byte (MENU_RELCUR) = starting relative selection
;	word (MENU_TITLE) = address of location of menu title/entries
;Output:
;	inputs adjusted if/as necessary
;	menu drawn
;	(MENU_ENTRIES) = address of start menu entries
InitMenu:
 call &CheckBounds
 ld ix,MENU_FLAG

 ld hl,(MENU_TITLE)
 xor a
 bit IXTitle,(ix+$00)
 jr z,IM_NoDispTitle	;jump if no title
 call &ClearAndDisp	;display title
 push hl
 ld bc,(MENU_Y)		;(B,C) = menu's (X,Y)
 call &XOR6Rows		;highlight title
 pop hl
 ld a,1
IM_NoDispTitle:
 ld (MENU_ENTRIES),hl	;save address of menu entries
 bit IXInsNums,(ix+$00)
 jr z,IM_NoInsNums

 push af
 ld a,(MENU_LENGTH)
 ld b,a
 cp 10+1
 jr c,$+4
 ld b,10

 ld e,a
 inc e
 push bc
 push hl
 call &GetString
 pop de
 pop bc
 dec hl
 ld a,$30
InsNumsLoop:
 push bc
 or a
 sbc hl,de
 ld b,h
 ld c,l
 inc bc
 add hl,de
 ld d,h
 ld e,l
 inc de\ inc de\ inc de
 push de
 lddr
 inc hl
 ld (hl),a\ inc hl
 ld (hl),$3A\ inc hl
 ld (hl),$20
 push af
 xor a
 cpir
 pop af
 ex de,hl
 pop hl
 inc a
 pop bc
 djnz InsNumsLoop
 pop af
IM_NoInsNums:

 bit IXBorder,(ix+$00)
 ret z
 ld hl,(MENU_WIDTH)	;L = width; H = height
 add a,h			;A = 0 if no title, 1 if there is a title
 A_X_6C
 ld h,a
 ld bc,$0202
 add hl,bc
 ex de,hl			;D = height of box; E = width
 ld bc,(MENU_Y)			;(B,C) = (X,Y) of box
 dec b
 dec c
 xor a			;'or' border
 BOX_CALL(BOX_DRAWBOX)
 ret


ResetMenu:
 call &CheckCur		;set cursor in right range

 ld hl,MENU_HEIGHT
 ld d,(hl)			;D = number of entries to display
 inc d
 dec d
 ret z			;if no entries in menu, return
 inc hl
 inc hl
 ld a,(hl)
 inc hl
 sub (hl)
 inc a
 ld e,a			;E = ABSCUR - RELCUR + 1 (first entry to display)

 bit IXDown,(ix+$00)
 jr z,RM_CantDown
 push de
 call &DispDownArrow
 pop de
 dec d
RM_CantDown:
 xor a			;A = offset from top of menu to display 1st entry
 bit IXUp,(ix+$00)
 jr z,RM_CantUp
 push de
 call &DispUpArrow
 pop de
 ld a,1
 dec d
 inc e
RM_CantUp:
 ld c,a
 ld a,(MENU_FLAG)
 and $01
 add a,c

RM_DispNewEntry:
 push af
 push de
 call &FindAndDisp
 pop de			;D = counter; E = string entry to display
 pop af			;A = row offset
 inc a
 inc e
 dec d
 jr nz,RM_DispNewEntry
 ret


;Input:
;	byte (MENU_Y) = menu's Y
;	byte (MENU_X) = menu's X
;	byte (MENU_WIDTH) = width of menu
;	byte (MENU_RELCUR) = entry to highlight
;Output:
;	Row at (MENU_RELCUR) is highlighted
;	AF,BC,DE,HL destroyed
XORCur:
 ld a,(MENU_FLAG)
 and $01
 dec a
 add a,(ix+IXRelCur)
 A_X_6C
 ld bc,(MENU_Y)
 add a,c
 ld c,a
XOR6Rows:
 ld a,$02
 ld de,(MENU_WIDTH)
 ld d,6
 BOX_CALL(BOX_FILLBOX)
 ret
Clear6Rows:
 ld a,$01
 jr XOR6Rows+2



;Input:
;	E = menu string to get
;	word (MENU_ENTRIES) = address of menu entries
;Ouput:
;	A = 0
;	BC = 0 - (bytes between start of entries and found string)
;	E = 0
;	HL -> string number E
GetMenuString:
 ld hl,(MENU_ENTRIES)
GetString:
 dec e
 ret z
 xor a
 ld c,a
 ld b,a
GS_Loop:
 cpir
 dec e
 jr nz,GS_Loop
 ret


DispUpArrow:
 ld c,0
 ld a,$06			;$06 = ascii down arrow
DUA_Merge:
 push af
 ld a,(MENU_FLAG)
 and $01
 add a,c
 A_X_6C
 add a,(ix+IXY)
 ld c,a
 ld b,(ix+IXX)
 push bc
 call &Clear6Rows		;clear rows where up arrow goes
 pop hl
 ld a,(MENU_WIDTH)
 srl a
 dec a
 dec a
 add a,h
 ld h,l
 ld l,a
 ld ($8333),hl
 pop af
 push ix
 call M_CHARPUT		;display arrow
 pop ix
 ret
DispDownArrow:
 ld c,(ix+IXHeight)
 dec c
 ld a,$07			;$07 = ascii down arrow
 jr DUA_Merge


;Input:
;	A = number of rows down in menu to clear and display string
;	E = entry number string to display
;	(HL -> string to display)
;	(MENU_Y) = menu's Y
;	(MENU_X) = menu's X
;	(MENU_WIDTH) = width of menu
;Output:
;	HL -> next string
FindAndDisp:
 push af
 call &GetMenuString
 pop af
ClearAndDisp:
 A_X_6C
 ld bc,(MENU_Y)
 add a,c
 ld c,a
 push hl
 push bc
 call &Clear6Rows
 pop hl
 EX_HL
 bit 1,(ix+$00)
 jr z,CAD_NoBuff
 inc l
CAD_NoBuff:
 ld ($8333),hl
 pop hl
CAD_DispStringJump:
 jr DispStringNoForm

DispStringForm:
 ld a,(MENU_WIDTH)
 ld c,a
 ld a,(MENU_X)
 add a,c
 ld (FORM_RIGHTEDGE),a
 push ix
 FORM_CALL(FORM_DISPSTRING)
 pop ix
 ret

DispStringNoForm:
DSNF_Loop:
 ld a,(hl)
 inc hl
 or a
 ret z
 cp 230
 jr nc,DSNF_Tag
 push ix
 call M_CHARPUT
 pop ix
 jr DSNF_Loop
DSNF_Tag:
 cp TABX
 jr nz,DSNF_Loop
 inc hl
 jr DSNF_Loop



CurDown:
 ld hl,MENU_ABSCUR
 ld a,(hl)
 dec hl
 cp (hl)
 inc hl
 jr nc,CD_AtBottom	;if at end of menu, move cursor to top
 inc (hl)			;else, increase real selection
 inc hl
 inc (hl)			;increase display selection
 jp &ResetMenu
CD_AtBottom:
 xor a
CD_ResetSel:
 ld (hl),a
 inc hl
 ld (hl),a			;(MenuRealSel) = (MenuDispSel) = 0
 jp &ResetMenu		; (will be set to 1)


CurUp:
 ld hl,MENU_ABSCUR
 ld a,(hl)
 dec a
 jr z,CU_AtTop		;if already selecting first entry, can't go higher
 dec (hl)			;else, decrease real selection
 inc  hl
 dec (hl)
 jp &ResetMenu
CU_AtTop:
 ld a,$FF
 jr CD_ResetSel
